import json
import logging
import os
import traceback
from pathlib import Path

import platformdirs

from overcooked_simulator import ROOT_DIR
from overcooked_simulator.hooks import HookCallbackClass
from overcooked_simulator.overcooked_environment import Environment
from overcooked_simulator.utils import NumpyAndDataclassEncoder

log = logging.getLogger(__name__)


class FileRecorder(HookCallbackClass):
    def __init__(
        self,
        name: str,
        env: Environment,
        log_path: str = "USER_LOG_DIR/ENV_NAME/LOG_RECORD_NAME.jsonl",
        add_hook_ref: bool = False,
        **kwargs,
    ):
        super().__init__(name, env, **kwargs)
        self.add_hook_ref = add_hook_ref
        log_path = log_path.replace("ENV_NAME", env.env_name).replace(
            "LOG_RECORD_NAME", name
        )
        if log_path.startswith("USER_LOG_DIR/"):
            log_path = (
                Path(platformdirs.user_log_dir("overcooked_simulator"))
                / log_path[len("USER_LOG_DIR/") :]
            )
        elif log_path.startswith("ROOT_DIR/"):
            log_path = ROOT_DIR / log_path[len("ROOT_DIR/") :]
        else:
            log_path = Path(log_path)
        self.log_path = log_path
        log.info(f"Recorder record for {name} in file://{log_path}")
        os.makedirs(log_path.parent, exist_ok=True)

    def __call__(self, hook_ref: str, env: Environment, **kwargs):
        try:
            record = (
                json.dumps(
                    {
                        "env_time": env.env_time.isoformat(),
                        **kwargs,
                        **({"hook_ref": hook_ref} if self.add_hook_ref else {}),
                    },
                    cls=NumpyAndDataclassEncoder,
                )
                + "\n"
            )
            with open(self.log_path, "a") as log_file:
                log_file.write(record)
        except TypeError as e:
            traceback.print_exception(e)
            log.info(f"Not JSON serializable Record {kwargs}")
