from typing import Any

from overcooked_simulator.hooks import HookCallbackClass
from overcooked_simulator.overcooked_environment import Environment


class ScoreViaHooks(HookCallbackClass):
    def __init__(
        self,
        name: str,
        env: Environment,
        static_score: float = 0,
        score_map: dict[str, float] = None,
        score_on_specific_kwarg: str = None,
        kwarg_filter: dict[str, Any] = None,
        **kwargs,
    ):
        super().__init__(name, env, **kwargs)
        self.score_map = score_map
        self.static_score = static_score
        self.kwarg_filter = kwarg_filter
        self.score_on_specific_kwarg = score_on_specific_kwarg

    def __call__(self, hook_ref: str, env: Environment, **kwargs):
        if self.score_on_specific_kwarg:
            if kwargs[self.score_on_specific_kwarg] in self.score_map:
                self.env.increment_score(
                    self.score_map[kwargs[self.score_on_specific_kwarg]],
                    info=f"{hook_ref} - {kwargs[self.score_on_specific_kwarg]}",
                )
            else:
                self.env.increment_score(self.static_score, info=hook_ref)
        elif self.score_map and hook_ref in self.score_map:
            if self.kwarg_filter:
                if kwargs.items() <= self.kwarg_filter.items():
                    self.env.increment_score(
                        self.score_map[hook_ref],
                        info=f"{hook_ref} - {self.kwarg_filter}",
                    )
            else:
                self.env.increment_score(self.score_map[hook_ref], info=hook_ref)
        else:
            self.env.increment_score(self.static_score, info=hook_ref)
